import java.io.*;

// Implementation of the RCS client command protocol

public class Actor
{
    // view width
    public static int NARROW = 0;
    public static int NORMAL = 1;
    public static int WIDE   = 2;

    // view quality
    public static int LOW    = 0;
    public static int HIGH   = 1;


    // reference to the communication module
    private Communication communication;

    public Actor(Communication communication)
    {
        this.communication = communication;
    }

    // Connecting, reconnecting, and disconnecting
    public void init(String teamName, boolean goalie)
    {
        if (goalie == false) {
            communication.send(new String("(init "+teamName+"(version "+ServerConf.PROTOCOL_VERSION+"))"));
        } else {
            communication.send(new String("(init "+teamName+"(version "+ServerConf.PROTOCOL_VERSION+") (goalie))"));
        }
    }

    public void reconnect(String teamName, int unum)
    {
        communication.send(new String("(reconnect " + teamName + " " + unum + ")"));
    }

    public void bye()
    {
        communication.send("(bye)");
    }

    // client control
    public void catsch(double degree)
    {
        communication.send(new String("(catch " + degree + ")"));
    }

    public void changeView(int width, int quality)
    {
        communication.send(new String("(change_view " + width + " " + quality + ")"));
    }

    public void dash(double power)
    {
        communication.send(new String("(dash " + power + ")"));
    }

    public void kick(double power, double direction)
    {
        communication.send(new String("(kick " + power + " " + direction + ")"));
    }

    public void move(double x, double y)
    {
        communication.send(new String("(move " + x + " " + y + ")"));
    }

    public void say(String message)
    {
        communication.send(new String("(say " + message + ")"));
    }

    public void senseBody()
    {
        communication.send(new String("(sense_body)"));
    }

    public void score()
    {
        communication.send(new String("(score)"));
    }

    public void turn(double moment)
    {
        communication.send(new String("(turn " + moment + ")"));
    }

    public void turnNeck(double angle)
    {
        communication.send(new String("(turn_neck " + angle + ")"));
    }
}
