import java.io.*;
import java.net.*;

/*
  Communication

  The class handles UDP network traffic from/to the rcssserver.
*/

public class Communication
{
    // socket for incoming, outgoing messages
    private DatagramSocket socket;
    // IP address of the server
    InetAddress serverAddress;
    // port of the server
    private int serverPort;

    Communication(String serverName, int serverPort) {
        try {
            // construct datagram socket and
            // bind it to any available port on the local machine
            this.socket        = new DatagramSocket();
            // set target address
            this.serverAddress = InetAddress.getByName(serverName);
            // set target port
            this.serverPort    = serverPort;
        } catch (SocketException se) {
            System.err.println("Communication: Could not create datagram socket.");
            System.exit(1);
        } catch (UnknownHostException uhe) {
            System.err.println("Communication: Could not determine the server's IP address.");
            System.exit(1);
        }

    }

    // send a message string to the server
    public void send(String message) {
        try {
            // convert the message to a byte array
            byte[] buffer = message.getBytes();
            // create the packet from the buffer and set the server's IP address and port
            DatagramPacket packet =
                new DatagramPacket(buffer, buffer.length, serverAddress, serverPort);
            // send the packet
            socket.send(packet);
        } catch (IOException ioe) {
            System.err.println("Communication: Could not send datagram packet.");
        }
    }

    // receive a message string from the server
    public String receive() {
        // allocate buffer for server message
        byte[] buffer = new byte[ServerConf.MAX_DGRAM_SIZE];
        // create datagram packet for receiving
        DatagramPacket packet = new DatagramPacket(buffer, buffer.length);
        try {
            // receive packet, the method blocks until a datagram arrives
            socket.receive(packet);
        } catch (IOException ioe) {
            System.err.println("Commmunication: Could not receive datagram packet.");
        }
        // get the server port for the following actions
        serverPort = packet.getPort();
        // return the message as string
        return new String(packet.getData());
    }

}

