import java.util.*;


public class MyWorld
{
    public String myTeamName;

    public int side; // the side on which we play
    public int unum; // the player number
    public int playMode = ServerConf.UNKNOWN;

    public AbsolutePosition myPosition = null;
    public double absAngle;

    public int visualTime     = 0; // time of the last visual event
    public ObjectInfo theGoal = null;
    public ObjectInfo myGoal  = null;

    // dynamic objects
    public AbsolutePosition ballPos = null;
    public double ballTime = 0;
    public double distanceToBall = 200;
    public int unumToBall;

    public ObjectInfo[] myTeamMatesNum = new ObjectInfo[12];
    public AbsolutePosition[] myTeamMatesNumPos = new AbsolutePosition[12];
    public ObjectInfo[] myTeamMates    = new ObjectInfo[12];
    public AbsolutePosition[] myTeamMatesPos = new AbsolutePosition[12];

    public ObjectInfo[] myRivalsNum = new ObjectInfo[12];
    public AbsolutePosition[] myRivalsNumPos = new AbsolutePosition[12];
    public ObjectInfo[] myRivals    = new ObjectInfo[12];
    public AbsolutePosition[] myRivalsPos = new AbsolutePosition[12];

    // static objects
    public AbsolutePosition[] markerPosition =
           new AbsolutePosition[ObjectInfo.LAST_MARKER]; // flags, goals

    static HashMap nameMap = new HashMap();
    static {
        String[] markers = {
            "p", "b",
            "g l", "g r",
            "f c",
            "f l t", "f l b", "f c t", "f c b", "f r t", "f r b",
            "f p l t", "f p l c", "f p l b", "f p r t", "f p r c", "f p r b",
            "f g l t", "f g l b", "f g r t", "f g r b",
            "f l 0", "f r 0", "f t 0", "f b 0",
            "f t l 10", "f t l 20", "f t l 30", "f t l 40", "f t l 50",
            "f t r 10", "f t r 20", "f t r 30", "f t r 40", "f t r 50",
            "f b l 10", "f b l 20", "f b l 30", "f b l 40", "f b l 50",
            "f b r 10", "f b r 20", "f b r 30", "f b r 40", "f b r 50",
            "f l t 10", "f l t 20", "f l t 30", "f l b 10", "f l b 20", "f l b 30",
            "f r t 10", "f r t 20", "f r t 30", "f r b 10", "f r b 20", "f r b 30",
            "l l", "l r", "l t", "l b"
        };

        for (int i = 0; i < markers.length; i++) {
            nameMap.put(markers[i], new Integer(i));
        }
    }

    public MyWorld()
    {
        // positions are given in "left-right" playing direction

        // {{{ initialize marker positions
        markerPosition[ObjectInfo.GOAL_L]      = new AbsolutePosition(-52.5,   0.0);
        markerPosition[ObjectInfo.GOAL_R]      = new AbsolutePosition( 52.5,   0.0);
        markerPosition[ObjectInfo.FLAG_C]      = new AbsolutePosition(  0.0,   0.0);
        markerPosition[ObjectInfo.FLAG_L_T]    = new AbsolutePosition(-52.5, -34.0);
        markerPosition[ObjectInfo.FLAG_L_B]    = new AbsolutePosition(-52.5,  34.0);
        markerPosition[ObjectInfo.FLAG_C_T]    = new AbsolutePosition(  0.0, -34.0);
        markerPosition[ObjectInfo.FLAG_C_B]    = new AbsolutePosition(  0.0,  34.0);
        markerPosition[ObjectInfo.FLAG_R_T]    = new AbsolutePosition( 52.5, -34.0);
        markerPosition[ObjectInfo.FLAG_R_B]    = new AbsolutePosition( 52.5,  34.0);

        markerPosition[ObjectInfo.FLAG_P_L_T]  = new AbsolutePosition(-36.0, -20.175);
        markerPosition[ObjectInfo.FLAG_P_L_C]  = new AbsolutePosition(-36.0,   0.0);
        markerPosition[ObjectInfo.FLAG_P_L_B]  = new AbsolutePosition(-36.0,  20.175);
        markerPosition[ObjectInfo.FLAG_P_R_T]  = new AbsolutePosition( 36.0, -20.175);
        markerPosition[ObjectInfo.FLAG_P_R_C]  = new AbsolutePosition( 36.0,   0.0);
        markerPosition[ObjectInfo.FLAG_P_R_B]  = new AbsolutePosition( 36.0,  20.175);

        markerPosition[ObjectInfo.FLAG_G_L_T]  = new AbsolutePosition(-52.5,  -7.01);
        markerPosition[ObjectInfo.FLAG_G_L_B]  = new AbsolutePosition(-52.5,   7.01);
        markerPosition[ObjectInfo.FLAG_G_R_T]  = new AbsolutePosition( 52.5,  -7.01);
        markerPosition[ObjectInfo.FLAG_G_R_B]  = new AbsolutePosition( 52.5,   7.01);

        markerPosition[ObjectInfo.FLAG_L_0]    = new AbsolutePosition(-57.5,   0.0);
        markerPosition[ObjectInfo.FLAG_R_0]    = new AbsolutePosition( 57.5,   0.0);
        markerPosition[ObjectInfo.FLAG_T_0]    = new AbsolutePosition(  0.0, -39.0);
        markerPosition[ObjectInfo.FLAG_B_0]    = new AbsolutePosition(  0.0,  39.0);
        markerPosition[ObjectInfo.FLAG_T_L_10] = new AbsolutePosition(-10.0, -39.0);
        markerPosition[ObjectInfo.FLAG_T_L_20] = new AbsolutePosition(-20.0, -39.0);
        markerPosition[ObjectInfo.FLAG_T_L_30] = new AbsolutePosition(-30.0, -39.0);
        markerPosition[ObjectInfo.FLAG_T_L_40] = new AbsolutePosition(-40.0, -39.0);
        markerPosition[ObjectInfo.FLAG_T_L_50] = new AbsolutePosition(-50.0, -39.0);
        markerPosition[ObjectInfo.FLAG_T_R_10] = new AbsolutePosition( 10.0, -39.0);
        markerPosition[ObjectInfo.FLAG_T_R_20] = new AbsolutePosition( 20.0, -39.0);
        markerPosition[ObjectInfo.FLAG_T_R_30] = new AbsolutePosition( 30.0, -39.0);
        markerPosition[ObjectInfo.FLAG_T_R_40] = new AbsolutePosition( 40.0, -39.0);
        markerPosition[ObjectInfo.FLAG_T_R_50] = new AbsolutePosition( 50.0, -39.0);
        markerPosition[ObjectInfo.FLAG_B_L_10] = new AbsolutePosition(-10.0,  39.0);
        markerPosition[ObjectInfo.FLAG_B_L_20] = new AbsolutePosition(-20.0,  39.0);
        markerPosition[ObjectInfo.FLAG_B_L_30] = new AbsolutePosition(-30.0,  39.0);
        markerPosition[ObjectInfo.FLAG_B_L_40] = new AbsolutePosition(-40.0,  39.0);
        markerPosition[ObjectInfo.FLAG_B_L_50] = new AbsolutePosition(-50.0,  39.0);
        markerPosition[ObjectInfo.FLAG_B_R_10] = new AbsolutePosition( 10.0,  39.0);
        markerPosition[ObjectInfo.FLAG_B_R_20] = new AbsolutePosition( 20.0,  39.0);
        markerPosition[ObjectInfo.FLAG_B_R_30] = new AbsolutePosition( 30.0,  39.0);
        markerPosition[ObjectInfo.FLAG_B_R_40] = new AbsolutePosition( 40.0,  39.0);
        markerPosition[ObjectInfo.FLAG_B_R_50] = new AbsolutePosition( 50.0,  39.0);
        markerPosition[ObjectInfo.FLAG_L_T_10] = new AbsolutePosition(-57.5, -10.0);
        markerPosition[ObjectInfo.FLAG_L_T_20] = new AbsolutePosition(-57.5, -20.0);
        markerPosition[ObjectInfo.FLAG_L_T_30] = new AbsolutePosition(-57.5, -30.0);
        markerPosition[ObjectInfo.FLAG_L_B_10] = new AbsolutePosition(-57.5,  10.0);
        markerPosition[ObjectInfo.FLAG_L_B_20] = new AbsolutePosition(-57.5,  20.0);
        markerPosition[ObjectInfo.FLAG_L_B_30] = new AbsolutePosition(-57.5,  30.0);
        markerPosition[ObjectInfo.FLAG_R_T_10] = new AbsolutePosition( 57.5, -10.0);
        markerPosition[ObjectInfo.FLAG_R_T_20] = new AbsolutePosition( 57.5, -20.0);
        markerPosition[ObjectInfo.FLAG_R_T_30] = new AbsolutePosition( 57.5, -30.0);
        markerPosition[ObjectInfo.FLAG_R_B_10] = new AbsolutePosition( 57.5,  10.0);
        markerPosition[ObjectInfo.FLAG_R_B_20] = new AbsolutePosition( 57.5,  20.0);
        markerPosition[ObjectInfo.FLAG_R_B_30] = new AbsolutePosition( 57.5,  30.0);
        // }}}
    }

    private void forget(int threshold)
    {
        if (ballPos != null && (visualTime - ballTime) > threshold/2) {
            distanceToBall = 200;
            ballPos = null;
        }

        for (int i = 1; i <= 11; i++) {
            if (myTeamMatesNum[i] != null &&
                (visualTime - myTeamMatesNum[i].time) > threshold)
                myTeamMatesNum[i] = null;
            if (myTeamMates[i] != null &&
                (visualTime - myTeamMates[i].time) > threshold)
                myTeamMates[i] = null;
            if (myRivalsNum[i] != null &&
                (visualTime - myRivalsNum[i].time) > threshold)
                myRivalsNum[i] = null;
            if (myRivals[i] != null &&
                (visualTime - myRivals[i].time) > threshold)
                myRivals[i] = null;
        }
    }


    public void initEventReceived(InitEvent event)
    {
        side     = event.side;
        unum     = event.unum;
        playMode = event.playMode;

        // since the absolute positions of the markers are stored in "left-right"
        // direction we have to invert their signs in case of a different side

        if (side != ServerConf.LEFT_SIDE) {
            for (int i = 0; i < markerPosition.length; i++) {
                if (markerPosition[i] != null) {
                    markerPosition[i].x = (-1.0) * markerPosition[i].x;
                    markerPosition[i].y = (-1.0) * markerPosition[i].y;
                }
            }
        }
    }

    public void auralEventReceived(AuralEvent event)
    {
        if (event.sender.equals("referee")) {
            if (event.message.equals("kick_off_l")) {
                playMode = ServerConf.KICK_OFF_L;
            } else if (event.message.equals("kick_off_r")) {
                playMode = ServerConf.KICK_OFF_R;
            } else if (event.message.startsWith("goal_l")) {
                playMode = ServerConf.GOAL_L;
            } else if (event.message.startsWith("goal_r")) {
                playMode = ServerConf.GOAL_R;
            }
        } else if (event.message.startsWith("\"" + myTeamName +"_ball") &&
            !event.sender.startsWith("self")) {
            StringTokenizer tokenizer = new StringTokenizer(event.message, "_\"");
            // read team name
            String teamName = tokenizer.nextToken();
            if (teamName.equals(myTeamName)) {
                // skip ball
                tokenizer.nextToken();
                double posX = Double.parseDouble(tokenizer.nextToken());
                double posY = Double.parseDouble(tokenizer.nextToken());
                ballPos = new AbsolutePosition(posX, posY);
                double dist = Double.parseDouble(tokenizer.nextToken());
                if (ballTime == event.time) {
                    if (dist < distanceToBall)
                        distanceToBall = dist;
                } else {
                    ballTime = event.time;
                    distanceToBall = dist;
                }
                unumToBall = Integer.parseInt(tokenizer.nextToken());
            }
        }
    }

    public void visualEventReceived(VisualEvent event)
    {
        visualTime = event.time;
        forget(10);

        ObjectInfo nearestMarker = null;
        double minMarkerDistance = 100.0;

        ObjectInfo line = null;
        double lineDistance = 0.0;

        ObjectInfo theBall = null;

        for (int i = 0; i < event.objectInfoList.size(); i++) {
            ObjectInfo objectInfo = (ObjectInfo) event.objectInfoList.get(i);
            switch (objectInfo.objectClass) {
            case ObjectInfo.BALL :
                theBall  = objectInfo;
                ballTime = objectInfo.time;
                break;
            case ObjectInfo.GOAL:
            case ObjectInfo.FLAG:
                if (objectInfo.objectName != null) {
                    if (objectInfo.distance < minMarkerDistance) {
                        nearestMarker     = objectInfo;
                        minMarkerDistance = objectInfo.distance;
                    }
                }
                break;
            case ObjectInfo.LINE:
                if (objectInfo.objectName != null) {
                    // choose line with largest distance
                    // this helps in the case the player is outside the field
                    if (objectInfo.distance > lineDistance) {
                        line         = objectInfo;
                        lineDistance = objectInfo.distance;
                    }
                }
                break;
            case ObjectInfo.PLAYER:
                if (myPosition == null)
                    continue;

                if (objectInfo.teamName != null) {
                    if (objectInfo.teamName.equals(myTeamName)) {
                        // store team mates
                        if (objectInfo.unum != -1) {
                            myTeamMatesNum[objectInfo.unum] = objectInfo;
                            myTeamMatesNumPos[objectInfo.unum] =
                                calculateAbsoluteObjectPosition(objectInfo);
                        } else {
                            for (int j = 1; j <= 11; j++)
                                if (myTeamMates[j] == null) {
                                    myTeamMates[j] = objectInfo;
                                    myTeamMatesPos[j] =
                                        calculateAbsoluteObjectPosition(objectInfo);
                                }
                        } // else
                    } else {
                        // store others
                        if (objectInfo.unum != -1) {
                            myRivalsNum[objectInfo.unum] = objectInfo;
                            myRivalsNumPos[objectInfo.unum] =
                                calculateAbsoluteObjectPosition(objectInfo);
                        } else {
                            for (int j = 1; j <= 11; j++)
                                if (myRivals[j] == null) {
                                    myRivals[j] = objectInfo;
                                    myRivalsPos[j] =
                                        calculateAbsoluteObjectPosition(objectInfo);
                                }
                        } // else
                    }

                }
                break;
            }
        }

        if (line != null && nearestMarker != null) {
            myPosition = calculateMyPosition(line, nearestMarker);
            if (theBall != null) {
                ballPos = calculateAbsoluteObjectPosition(theBall);
            }
        }
    }

    public void bodyEventReceived(String message)
    {
        // System.out.println("My world: body event received");
    }

    public void update(Event event)
    {
        switch (event.type) {
        case Event.INIT_EVENT:
            initEventReceived((InitEvent) event);
            break;
        case Event.VISUAL_EVENT:
            visualEventReceived((VisualEvent) event);
            break;
        case Event.AURAL_EVENT:
            auralEventReceived((AuralEvent) event);
            break;
        case Event.BODY_EVENT:
            // bodyEventReceived((BodyEvent) event);
            break;
        }
    }


    public final int getSide()
    {
        return side;
    }

    public final int getUnum()
    {
        return unum;
    }

    public final int getPlayMode()
    {
        return playMode;
    }

    private AbsolutePosition calculateMyPosition(ObjectInfo currentLine,
                                                 ObjectInfo nearestFlag)
    {
        // calculate absolute view point angle. 0 points to the north.

        // name of a line is e.g. "l l" or "l t"
        switch (currentLine.objectName.charAt(2)) {
        case 't':
            if ((side * currentLine.direction) < 0) {
                absAngle = 180 - currentLine.direction;
            } else {
                absAngle = 0 - currentLine.direction;
            }
            break;
        case 'b':
            if ((side * currentLine.direction) < 0) {
                absAngle = 0 - currentLine.direction;
            } else {
                absAngle = 180 - currentLine.direction;
            }
            break;
        case 'l':
            if ((side * currentLine.direction) < 0) {
                absAngle =  90 - currentLine.direction;
            } else {
                absAngle = 270 - currentLine.direction;
            }
            break;
        case 'r':
            if ((side * currentLine.direction) < 0) {
                absAngle = 270 - currentLine.direction;
            } else {
                absAngle =  90 - currentLine.direction;
            }
            break;
        default:
            // this should not happen
            System.out.println("Could not match line");
            System.exit(1);
        }

        if (absAngle > 180)
            absAngle -= 360;
        else if (absAngle < -180)
            absAngle += 360;

        // calculate absolute polar vector to nearestFlag
        double absFlagAngle = absAngle + nearestFlag.direction;

        // transform the vector from polar to absolute vectors
        double x = nearestFlag.distance * Math.cos(Math.toRadians(absFlagAngle));
        double y = nearestFlag.distance * Math.sin(Math.toRadians(absFlagAngle));

        // transform the vector to absolute coordinates by taking the
        // known absolute position of the seen flag as a support vector
        Object id = nameMap.get(nearestFlag.objectName);
        if (id == null) {
            System.out.println("Could not find " + nearestFlag.objectName);
            System.exit(1);
        }
        AbsolutePosition markerPos = markerPosition[((Integer) id).intValue()];

        double xPos = markerPos.x - x;
        double yPos = markerPos.y - y;

        // System.out.println("Your position is: x: " + xPos + " y: " + yPos);

        return new AbsolutePosition(xPos, yPos);
    }

    public double getOffsideLine()
    {
        double playerPosRight = -100;

        for (int i = 1; i <= 11; i++) {
            if (myRivalsNum[i] != null && playerPosRight < myRivalsNumPos[i].x) {
                playerPosRight = myRivalsNumPos[i].x;
            }
            if (myRivals[i] != null && playerPosRight < myRivalsPos[i].x) {
                playerPosRight = myRivalsPos[i].x;
            }
        }


        double playerPos = -100;
        if (playerPosRight != -100) {
            for (int i = 1; i <= 11; i++) {
                if (myRivalsNum[i] != null &&
                    playerPos < myRivalsNumPos[i].x &&
                    playerPosRight > myRivalsNumPos[i].x) {
                    playerPos = myRivalsNumPos[i].x;
                }
                if (myRivals[i] != null &&
                    playerPos < myRivalsPos[i].x &&
                    playerPosRight > myRivalsPos[i].x) {
                    playerPos = myRivalsPos[i].x;
                }
            }
        }

        if (playerPos < 0) {
            playerPos = -1.0;
        }

        return playerPos;
    }


    public double calculateDistance(AbsolutePosition pos1, AbsolutePosition pos2)
    {
        double dx = pos1.x - pos2.x;
        double dy = pos1.y - pos2.y;

        return Math.sqrt(dx*dx + dy*dy);
    }

    public boolean amIClosestToTheBall()
    {
        if (ballPos == null || myPosition == null)
            return false;

        double myDist = calculateDistance(ballPos, myPosition);

        boolean hasTeamMates = false;

        boolean result = false;
        for (int i = 1; i <= 11 && !hasTeamMates; i++) {
            if (myTeamMatesNum[i] != null &&
                calculateDistance(ballPos, myTeamMatesNumPos[i]) < myDist) {
                hasTeamMates = true;
                result       = false;
            }
            if (myTeamMates[i] != null &&
                calculateDistance(ballPos, myTeamMatesPos[i]) < myDist) {
                hasTeamMates = true;
                result       = false;
            }
        }

        if (!hasTeamMates) {
            if (myDist < distanceToBall)
                result = true;
            else
                result = false;
        }

        return result;
    }


    // calculates the absolute position of an object on the field
    public AbsolutePosition calculateAbsoluteObjectPosition(ObjectInfo objectInfo)
    {
        double absObjAngle = absAngle + objectInfo.direction;
        double x = objectInfo.distance * Math.cos(Math.toRadians(absObjAngle));
        double y = objectInfo.distance * Math.sin(Math.toRadians(absObjAngle));

        return new AbsolutePosition(x + myPosition.x, myPosition.y + y);
    }

    // calculates the relative position of an object on the field
    public RelativePosition calculateRelativeObjectPosition(AbsolutePosition objectPosition)
    {
        double dx = objectPosition.x - myPosition.x;
        double dy = objectPosition.y - myPosition.y;
        double distance = Math.sqrt(dx*dx + dy*dy);

        double angle = 0.0;

        if (dx < 0)
            angle = Math.atan(dy/dx) + Math.PI;
        else if (dx > 0)
            angle = Math.atan(dy/dx);
        else if (dx == 0 && dy > 0)
            angle = Math.PI * 0.5;
        else if (dx == 0 && dy < 0)
            angle = -Math.PI * 0.5;
        else {
            angle = 0.0;
        }
        angle = Math.toDegrees(angle);


        if (angle > 180)
            angle -=360;
        else if (angle < -180)
            angle +=360;

        return new RelativePosition(distance, angle);
    }


}
