public class Player extends Thread
{
    private Actor      actor;
    private Sensor     sensor;
    private MyWorld    myWorld;
    private MyStrategy myStrategy;

    private String  teamName;
    private boolean position;

    public Player(String servername, int serverport, String teamName, boolean position)
    {
        this.teamName = teamName;
        this.position = position;

        Communication communication = new Communication(servername, serverport);
        actor   = new Actor(communication);
        sensor  = new Sensor(communication);
        myWorld = new MyWorld();
        myWorld.myTeamName = teamName;

        // could choose different strategy module depending on the position parameter
        // e.g. goalie, midfield
        myStrategy = new ZoneStrategy(actor, sensor, myWorld);
    }

    public void run()
    {
        Event event = null;
        // connect to server
        actor.init(teamName, position);
        // and play
        while (true) {
            // receive event - block until one arrives
            event = sensor.receive();
            if (event != null) {
                // update world modell
                myWorld.update(event);
                // action ...
                if (event instanceof BodyEvent)
                    myStrategy.play();
            }
        }
    }
}
