import java.io.*;
import java.util.*;

// Implementation of the RCS client sensor protocol

public class Sensor
{
    private Communication communication;

    public Sensor(Communication communication)
    {
        this.communication = communication;
    }

    public Event receive()
    {
        // receive server message - block until one arrives
        String msg = communication.receive();

        Event event = null;
        if (msg.startsWith("(init")) {
            // it's an init event
            event = new InitEvent(msg);
        } else if (msg.startsWith("(hear")) {
            // it's an aural ("hear") event
            event = new AuralEvent(msg);
        } else if (msg.startsWith("(see")) {
            // it's a visual ("see") or a body ("sense body") event
            event = new VisualEvent(msg);
        } else if (msg.startsWith("(sense_body")) {
            // it's a body ("sense_body") event
            event = new BodyEvent(msg);
        } else {
            // error event or server event
            // * should handle server params here *
            System.out.println ("unknown event: " + msg);
        }

        return event;
    }

}
