/*
  simple strategy
  "look for the ball, run to it, and kick it in the direction of goal"
*/


public class SimpleStrategy implements MyStrategy
{
    private Actor   actor;
    private Sensor  sensor;
    private MyWorld myWorld;

    static final int NOT_CONNECTED  = 0;
    static final int BEFORE_KICKOFF = 1;
    static final int PLAY           = 2;

    private int mode;
    private int lastTime = 0;

    public SimpleStrategy(Actor actor, Sensor sensor, MyWorld myWorld)
    {
        this.actor   = actor;
        this.sensor  = sensor;
        this.myWorld = myWorld;

        this.mode    = NOT_CONNECTED;
    }

    public void play()
    {
        switch (mode) {
        case NOT_CONNECTED:
            if (myWorld.getPlayMode() == ServerConf.BEFORE_KICK_OFF) {
                // set random position on the field
                // own side has negative x coordinates
                double posX = -Math.random() * ServerConf.POSX_MAX;
                // top is negative, bottom positive
                double posY = Math.random() * 2 * ServerConf.POSY_MAX - ServerConf.POSY_MAX;
                // actor.move(posX, posY);
                actor.move(-10, -10);
                mode = BEFORE_KICKOFF;
            }
            break;

        case BEFORE_KICKOFF:
            if (lastTime != myWorld.visualTime) {
                lastTime = myWorld.visualTime;
                mode = PLAY;
            }
            break;

        case PLAY:
            if (lastTime == myWorld.visualTime)
                return;

            // already found the ball?
            if (myWorld.ballPos == null) {
                // no, look around
                actor.turn(40);
            } else {
                // we found the ball
                if (dashToPosition(myWorld.ballPos, 0.75)) {
                    RelativePosition relPos = new RelativePosition();
                    if (myWorld.side == ServerConf.LEFT_SIDE)
                        relPos = myWorld.calculateRelativeObjectPosition(
                            myWorld.markerPosition[ObjectInfo.GOAL_R]);
                    else
                        relPos = myWorld.calculateRelativeObjectPosition(
                            myWorld.markerPosition[ObjectInfo.GOAL_L]);
                    double angleToTurn = getSmallestAngle(relPos.angle - myWorld.absAngle);
                    if (Math.abs(angleToTurn) > 1.0) {
                        actor.turn(angleToTurn);
                        myWorld.absAngle = normalizeAngle(myWorld.absAngle + angleToTurn);
                    } else {
                        actor.kick(100, angleToTurn);
                    }
                }
            }
            break;
        }
    }

    private final static double getSmallestAngle(double angle)
    {
        if (angle > 180)
            angle -=360;
        else if (angle < -180)
            angle +=360;

        return angle;
    }
    
    private final static double normalizeAngle(double angle)
    {
        if (angle >= 360)
            angle -= 360;
        else if (angle <= -360)
            angle += 360;

        return angle;
    }
    
    
    private boolean dashToPosition(AbsolutePosition absPos, double maxDistance)
    {
        RelativePosition pos = myWorld.calculateRelativeObjectPosition(absPos);

        if (pos.distance < maxDistance)
            return true;

        double angleToTurn = getSmallestAngle(pos.angle - myWorld.absAngle);

        if (Math.abs(angleToTurn) > 10.0) {
            actor.turn(angleToTurn);
            myWorld.absAngle = normalizeAngle(myWorld.absAngle + angleToTurn);
        } else {
            actor.dash(50);
        }

        return false;
    }


}
