import java.util.*;

public class VisualEvent extends Event
{
    public ArrayList objectInfoList = new ArrayList();

    public VisualEvent(String message)
    {
        type = Event.VISUAL_EVENT;
        StringTokenizer	tokenizer = new StringTokenizer(message, "()");

        // extract the time stamp
        // - nextToken returns something like "see 15 "
        // - use substring and trim whitespaces
        time = Integer.parseInt(tokenizer.nextToken().substring(4).trim());

        while (tokenizer.hasMoreTokens()) {
            // create a new object info
            ObjectInfo objectInfo = new ObjectInfo();
            objectInfo.time = time;
            // get the object name, e.g. "f g r t" or "p "TEAM" 1 goalie"
            objectInfo.objectName = tokenizer.nextToken();

            switch (objectInfo.objectName.charAt(0)) {
            case 'p':
                parsePlayerInfo(objectInfo, tokenizer);
                break;
            case 'b':
                objectInfo.objectClass = ObjectInfo.BALL;
                parseBallInfo(objectInfo, tokenizer);
                break;
            case 'f':
                objectInfo.objectClass = ObjectInfo.FLAG;
                parseMarkerInfo(objectInfo, tokenizer);
                break;
            case 'g':
                objectInfo.objectClass = ObjectInfo.GOAL;
                parseMarkerInfo(objectInfo, tokenizer);
                break;
            case 'l':
                objectInfo.objectClass = ObjectInfo.LINE;
                parseMarkerInfo(objectInfo, tokenizer);
                break;
            }

            objectInfoList.add(objectInfo);
        }
    }

    final private void parsePlayerInfo(ObjectInfo objectInfo, StringTokenizer tokenizer)
    {
        // parse player infos from the object name
        StringTokenizer st1 = new StringTokenizer(objectInfo.objectName, " ");
        st1.nextToken(); // skip "p"
        if (st1.hasMoreTokens()) {
            // there is a team name, e.g. as in (p "TEAM")
            String tmpTeamName = st1.nextToken(); // extract quoted name
            // store team name without quotes
            objectInfo.teamName = tmpTeamName.substring(1, tmpTeamName.length() - 1);
        }
        if (st1.hasMoreTokens()) {
            // there is a uniform team number, e.g. as in (p "TEAM" 10)
            objectInfo.unum = Integer.parseInt(st1.nextToken());
        }
        if (st1.hasMoreTokens()) {
            // there is a "goalie" tag, e.g. as in (p "TEAM" 1 goalie)
            objectInfo.goalie = true;
        }

        // now parse the position and movement infos
        StringTokenizer st2 = new StringTokenizer(tokenizer.nextToken(), " ");
        objectInfo.distance    = Double.parseDouble(st2.nextToken());
        objectInfo.direction   = Double.parseDouble(st2.nextToken());
        if (st2.hasMoreTokens()) {
            // there are a movement infos
            objectInfo.distChange = Double.parseDouble(st2.nextToken());
            objectInfo.dirChange = Double.parseDouble(st2.nextToken());
        }
        if (st2.hasMoreTokens()) {
            // there are facing infos
            objectInfo.headFacingDir = Double.parseDouble(st2.nextToken());
            objectInfo.bodyFacingDir = Double.parseDouble(st2.nextToken());
        }
    }

    final private void parseBallInfo(ObjectInfo objectInfo, StringTokenizer tokenizer)
    {
        StringTokenizer st     = new StringTokenizer(tokenizer.nextToken(), " ");
        objectInfo.distance    = Double.parseDouble(st.nextToken());
        objectInfo.direction   = Double.parseDouble(st.nextToken());
        if (st.hasMoreTokens()) {
            // there are a movement infos
            objectInfo.distChange = Double.parseDouble(st.nextToken());
            objectInfo.dirChange  = Double.parseDouble(st.nextToken());
        }
    }

    final private void parseMarkerInfo(ObjectInfo objectInfo, StringTokenizer tokenizer)
    {
        StringTokenizer st     = new StringTokenizer(tokenizer.nextToken(), " ");
        objectInfo.distance    = Double.parseDouble(st.nextToken());
        objectInfo.direction   = Double.parseDouble(st.nextToken());
    }

}
