/*
  sophisticated strategy :->
*/


public class ZoneStrategy implements MyStrategy
{
    private Actor   actor;
    private Sensor  sensor;
    private MyWorld myWorld;

    private int lastTime = 0;

    private static AbsolutePosition[] startPos = new AbsolutePosition[12];
    private static AbsolutePosition[] gamePos = new AbsolutePosition[12];
    static
    {
        startPos[1]  = new AbsolutePosition(-50, 0);
        startPos[2]  = new AbsolutePosition(-10, 0);
        startPos[3]  = new AbsolutePosition(-20, 0);
        startPos[4]  = new AbsolutePosition(-40, -5);
        startPos[5]  = new AbsolutePosition(-40, 5);
        startPos[6]  = new AbsolutePosition(-10, -15);
        startPos[7]  = new AbsolutePosition(-20, -15);
        startPos[8]  = new AbsolutePosition(-40, -15);
        startPos[9]  = new AbsolutePosition(-10, 15);
        startPos[10] = new AbsolutePosition(-20, 15);
        startPos[11] = new AbsolutePosition(-40, 15);

        gamePos[1]   = new AbsolutePosition(-50, 0);
        gamePos[2]   = new AbsolutePosition( 20, 0);
        gamePos[3]   = new AbsolutePosition(-10, 0);
        gamePos[4]   = new AbsolutePosition(-40, -3);
        gamePos[5]   = new AbsolutePosition(-40,  3);
        gamePos[6]   = new AbsolutePosition( 20, -15);
        gamePos[7]   = new AbsolutePosition(-10, -15);
        gamePos[8]   = new AbsolutePosition(-40, -10);
        gamePos[9]   = new AbsolutePosition( 20, 15);
        gamePos[10]  = new AbsolutePosition(-10, 15);
        gamePos[11]  = new AbsolutePosition(-40, 10);
    }


    public ZoneStrategy(Actor actor, Sensor sensor, MyWorld myWorld)
    {
        this.actor   = actor;
        this.sensor  = sensor;
        this.myWorld = myWorld;
    }

    public void play()
    {
        switch (myWorld.getPlayMode()) {
        case ServerConf.BEFORE_KICK_OFF:
        case ServerConf.GOAL_L:
        case ServerConf.GOAL_R:
            actor.move(startPos[myWorld.unum].x, startPos[myWorld.unum].y);
            break;

        default:
            // already found the ball?
            if (myWorld.ballPos == null) {
                // no, look around
                actor.turn(40);
            } else {
                // we found the ball
                double dist     = myWorld.calculateDistance(myWorld.ballPos,
                                                        myWorld.myPosition);
                boolean closest = myWorld.amIClosestToTheBall();

                if (closest || dist < 10.0) {
                    if (closest)
                        actor.say(myWorld.myTeamName + "_ball_"
                                  + myWorld.ballPos.x + "_" + myWorld.ballPos.y
                                  + "_" + dist + "_" + myWorld.unum);

                    if (dashToPosition(myWorld.ballPos, 80, 0.75)) {
                        AbsolutePosition goalPos = null;
                        if (myWorld.side == ServerConf.LEFT_SIDE)
                            goalPos = myWorld.markerPosition[ObjectInfo.GOAL_R];
                        else
                            goalPos = myWorld.markerPosition[ObjectInfo.GOAL_L];
                        kickToPosition(goalPos);
                    }
                } else {
                    AbsolutePosition posToGo = new AbsolutePosition();
                    posToGo.x = gamePos[myWorld.unum].x;
                    posToGo.y = gamePos[myWorld.unum].y;

                    double offsideLine = myWorld.getOffsideLine();
                    if (offsideLine < 100 && posToGo.x > offsideLine)
                        posToGo.x = offsideLine;
                    if (dashToPosition(posToGo, 50, 1.0)) {
                        if (dist > 25.0)
                            actor.turn(45);
                        else
                            turnTo(myWorld.myPosition, myWorld.ballPos);
                    }

                }

            }
            break;
        }
    }

    private final static double getSmallestAngle(double angle)
    {
        if (angle > 180)
            angle -=360;
        else if (angle < -180)
            angle +=360;

        return angle;
    }

    private final static double normalizeAngle(double angle)
    {
        if (angle >= 360)
            angle -= 360;
        else if (angle <= -360)
            angle += 360;

        return angle;
    }

    private void turnTo(AbsolutePosition myPosition, AbsolutePosition absPos)
    {
        RelativePosition pos = myWorld.calculateRelativeObjectPosition(absPos);

        double angleToTurn = getSmallestAngle(pos.angle - myWorld.absAngle);

        if (Math.abs(angleToTurn) > 1.0) {
            actor.turn(angleToTurn);
            myWorld.absAngle = normalizeAngle(myWorld.absAngle + angleToTurn);
        }
    }


    private boolean dashToPosition(AbsolutePosition absPos, double speed, double maxDistance)
    {
        RelativePosition pos = myWorld.calculateRelativeObjectPosition(absPos);

        if (pos.distance < maxDistance)
            return true;

        double angleToTurn = getSmallestAngle(pos.angle - myWorld.absAngle);

        if (Math.abs(angleToTurn) > 10.0) {
            actor.turn(angleToTurn);
            myWorld.absAngle = normalizeAngle(myWorld.absAngle + angleToTurn);
        } else {
            actor.dash(speed);
        }

        return false;
    }

    private boolean kickToPosition(AbsolutePosition absPos)
    {
        RelativePosition relPos = myWorld.calculateRelativeObjectPosition(absPos);
        double angleToTurn = getSmallestAngle(relPos.angle - myWorld.absAngle);
        if (Math.abs(angleToTurn) > 45.0) {
            actor.kick(100, angleToTurn);
        } else {
            actor.kick(100, angleToTurn);
        }

        return true;
    }

}
